﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Optimization.Views {
  [View("Independent Random Restarter View")]
  [Content(typeof(IndepdentRandomRestarter), IsDefaultView = true)]
  public partial class IndependentRandomRestarterView : IOptimizerView {
    protected TypeSelectorDialog algorithmTypeSelectorDialog;
    protected virtual bool SuppressEvents { get; set; }

    public new IndepdentRandomRestarter Content {
      get { return (IndepdentRandomRestarter)base.Content; }
      set { base.Content = value; }
    }

    public IndependentRandomRestarterView() {
      InitializeComponent();
      terminationComboBox.Items.AddRange(Enum.GetValues(typeof(TerminationCriterium)).Cast<object>().ToArray());
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (algorithmTypeSelectorDialog != null) algorithmTypeSelectorDialog.Dispose();
        if (components != null) components.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void DeregisterContentEvents() {
      Content.PropertyChanged -= Content_PropertyChanged;
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.PropertyChanged += Content_PropertyChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      SuppressEvents = true;
      try {
        if (Content == null) {
          maxExecutionTimeTextBox.Text = String.Empty;
          maxEvaluationsTextBox.Text = String.Empty;
          moveCostPerSolutionTextBox.Text = String.Empty;
          targetValueTextBox.Text = String.Empty;
          maximizationCheckBox.CheckState = CheckState.Indeterminate;
          terminationComboBox.SelectedIndex = -1;

          algorithmViewHost.Content = null;
          currentRunView.Content = null;
          runsView.Content = null;
        } else {
          maxExecutionTimeTextBox.Text = TimeSpanHelper.FormatNatural(Content.MaximumExecutionTime);
          maxEvaluationsTextBox.Text = Content.MaximumEvaluations.ToString();
          moveCostPerSolutionTextBox.Text = Content.MoveCostPerSolution.ToString(CultureInfo.CurrentCulture.NumberFormat);
          targetValueTextBox.Text = Content.TargetValue.ToString(CultureInfo.CurrentCulture.NumberFormat);
          maximizationCheckBox.Checked = Content.Maximization;
          terminationComboBox.SelectedItem = Content.TerminationCriterium;

          algorithmViewHost.Content = Content.Algorithm;
          currentRunView.Content = Content.CurrentRun;
          runsView.Content = Content.Runs;
        }
      } finally { SuppressEvents = false; }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      maxExecutionTimeTextBox.Enabled = Content != null && !ReadOnly && !Locked;
      maxEvaluationsTextBox.Enabled = Content != null && !ReadOnly && !Locked;
      moveCostPerSolutionTextBox.Enabled = Content != null && !ReadOnly && !Locked;
      targetValueTextBox.Enabled = Content != null && !ReadOnly && !Locked;
      maximizationCheckBox.Enabled = Content != null && !ReadOnly && !Locked;
      terminationComboBox.Enabled = Content != null && !ReadOnly && !Locked;
      newAlgorithmButton.Enabled = Content != null && !ReadOnly;
      openAlgorithmButton.Enabled = Content != null && !ReadOnly;
      algorithmViewHost.Enabled = Content != null;
      currentRunView.Enabled = Content != null;
      runsView.Enabled = Content != null;
    }

    protected override void OnClosed(FormClosedEventArgs e) {
      if ((Content != null) && (Content.ExecutionState == ExecutionState.Started)) {
        //The content must be stopped if no other view showing the content is available
        var optimizers = MainFormManager.MainForm.Views.OfType<IContentView>().Where(v => v != this).Select(v => v.Content).OfType<IAlgorithm>();
        if (!optimizers.Contains(Content.Algorithm)) {
          var nestedOptimizers = optimizers.SelectMany(opt => opt.NestedOptimizers);
          if (!nestedOptimizers.Contains(Content)) Content.Stop();
        }
      }
      base.OnClosed(e);
    }

    #region Event Handlers
    #region Content events
    private void Content_PropertyChanged(object sender, PropertyChangedEventArgs e) {
      SuppressEvents = true;
      try {
        switch (e.PropertyName) {
          case "TerminationCriterium": terminationComboBox.SelectedItem = Content.TerminationCriterium; break;
          case "MaximumExecutionTime": maxExecutionTimeTextBox.Text = TimeSpanHelper.FormatNatural(Content.MaximumExecutionTime); break;
          case "MaximumEvaluations": maxEvaluationsTextBox.Text = Content.MaximumEvaluations.ToString(); break;
          case "TargetValue": targetValueTextBox.Text = Content.TargetValue.ToString(CultureInfo.CurrentCulture.NumberFormat); break;
          case "Maximization": maximizationCheckBox.Checked = Content.Maximization; break;
          case "MoveCostPerSolution": moveCostPerSolutionTextBox.Text = Content.MoveCostPerSolution.ToString(CultureInfo.CurrentCulture.NumberFormat); break;
          case "Algorithm": algorithmViewHost.Content = Content.Algorithm; break;
          case "CurrentRun": currentRunView.Content = Content.CurrentRun; break;
          case "Runs": runsView.Content = Content.Runs; break;
        }
      } finally { SuppressEvents = false; }
    }
    #endregion

    #region Control events
    private void maxExecutionTimeTextBox_Validating(object sender, CancelEventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, CancelEventArgs>)maxExecutionTimeTextBox_Validating, sender, e);
        return;
      }
      TimeSpan ts;
      if (!TimeSpanHelper.TryGetFromNaturalFormat(maxExecutionTimeTextBox.Text, out ts)) {
        e.Cancel = !maxExecutionTimeTextBox.ReadOnly && maxExecutionTimeTextBox.Enabled;
        errorProvider.SetError(maxExecutionTimeTextBox, "Please enter a valid time span, e.g. 20 seconds ; 45s ; 4min ; 1h ; 3 hours ; 2 days ; 4d");
      } else {
        Content.MaximumExecutionTime = ts;
        e.Cancel = false;
        errorProvider.SetError(maxExecutionTimeTextBox, null);
      }
    }

    private void maxEvaluationsTextBox_Validating(object sender, CancelEventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, CancelEventArgs>)maxEvaluationsTextBox_Validating, sender, e);
        return;
      }
      int value;
      if (!int.TryParse(maxEvaluationsTextBox.Text, out value)) {
        e.Cancel = !maxEvaluationsTextBox.ReadOnly && maxEvaluationsTextBox.Enabled;
        errorProvider.SetError(maxEvaluationsTextBox, "Please enter a valid integer number.");
      } else {
        Content.MaximumEvaluations = value;
        e.Cancel = false;
        errorProvider.SetError(maxEvaluationsTextBox, null);
      }
    }

    private void moveCostPerSolutionTextBox_Validating(object sender, CancelEventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, CancelEventArgs>)moveCostPerSolutionTextBox_Validating, sender, e);
        return;
      }
      double value;
      if (!double.TryParse(moveCostPerSolutionTextBox.Text, out value)) {
        e.Cancel = !moveCostPerSolutionTextBox.ReadOnly && moveCostPerSolutionTextBox.Enabled;
        errorProvider.SetError(moveCostPerSolutionTextBox, "Please enter a valid integer number.");
      } else {
        Content.MoveCostPerSolution = value;
        e.Cancel = false;
        errorProvider.SetError(moveCostPerSolutionTextBox, null);
      }
    }

    private void targetValueTextBox_Validating(object sender, CancelEventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, CancelEventArgs>)targetValueTextBox_Validating, sender, e);
        return;
      }
      double value;
      if (!double.TryParse(targetValueTextBox.Text, out value)) {
        e.Cancel = !targetValueTextBox.ReadOnly && targetValueTextBox.Enabled;
        errorProvider.SetError(targetValueTextBox, "Please enter a valid integer number.");
      } else {
        Content.TargetValue = value;
        e.Cancel = false;
        errorProvider.SetError(targetValueTextBox, null);
      }
    }

    private void maximizationCheckBox_CheckedChanged(object sender, EventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, EventArgs>)maximizationCheckBox_CheckedChanged, sender, e);
        return;
      }
      Content.Maximization = maximizationCheckBox.Checked;
    }

    private void terminationComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (SuppressEvents) return;
      if (InvokeRequired) {
        Invoke((Action<object, EventArgs>)terminationComboBox_SelectedIndexChanged, sender, e);
        return;
      }
      Content.TerminationCriterium = (TerminationCriterium)terminationComboBox.SelectedItem;
    }

    private void newAlgorithmButton_Click(object sender, EventArgs e) {
      if (algorithmTypeSelectorDialog == null) {
        algorithmTypeSelectorDialog = new TypeSelectorDialog { Caption = "Select Algorithm" };
        algorithmTypeSelectorDialog.TypeSelector.Caption = "Available Algorithms";
        algorithmTypeSelectorDialog.TypeSelector.Configure(typeof(IAlgorithm)
          , showNotInstantiableTypes: false, showGenericTypes: false);
      }
      if (algorithmTypeSelectorDialog.ShowDialog(this) == DialogResult.OK) {
        try {
          Content.Algorithm = (IAlgorithm)algorithmTypeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
        } catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(this, ex);
        }
      }
    }

    private void openAlgorithmButton_Click(object sender, EventArgs e) {
      openFileDialog.Title = "Open Algorithm";
      if (openFileDialog.ShowDialog(this) == DialogResult.OK) {
        newAlgorithmButton.Enabled = openAlgorithmButton.Enabled = false;
        algorithmViewHost.Enabled = false;

        ContentManager.LoadAsync(openFileDialog.FileName, delegate(IStorableContent content, Exception error) {
          try {
            if (error != null) throw error;
            var algorithm = content as IAlgorithm;
            if (algorithm == null)
              MessageBox.Show(this, "The selected file does not contain an algorithm.", "Invalid File", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
              Content.Algorithm = algorithm;
          } catch (Exception ex) {
            ErrorHandling.ShowErrorDialog(this, ex);
          } finally {
            Invoke(new Action(delegate() {
              algorithmViewHost.Enabled = true;
              newAlgorithmButton.Enabled = openAlgorithmButton.Enabled = true;
            }));
          }
        });
      }
    }

    private void algorithmTabPage_DragEnterOver(object sender, DragEventArgs e) {
      e.Effect = DragDropEffects.None;
      var alg = e.Data.GetData(Constants.DragDropDataFormat) as IAlgorithm;
      if (!ReadOnly && alg != null) {
        if (!typeof(ISingleObjectiveHeuristicOptimizationProblem).IsAssignableFrom(alg.ProblemType))
          return;
        if ((e.KeyState & 32) == 32) e.Effect = DragDropEffects.Link;  // ALT key
        else if ((e.KeyState & 4) == 4) e.Effect = DragDropEffects.Move;  // SHIFT key
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Copy)) e.Effect = DragDropEffects.Copy;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Move)) e.Effect = DragDropEffects.Move;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Link)) e.Effect = DragDropEffects.Link;
      }
    }

    private void algorithmTabPage_DragDrop(object sender, DragEventArgs e) {
      if (e.Effect != DragDropEffects.None) {
        var algorithm = e.Data.GetData(Constants.DragDropDataFormat) as IAlgorithm;
        if (e.Effect.HasFlag(DragDropEffects.Copy)) algorithm = (IAlgorithm)algorithm.Clone();
        Content.Algorithm = algorithm;
      }
    }
    #endregion
    #endregion
  }
}
